# frozen_string_literal: true

# Send webhook to Read the docs according to https://docs.readthedocs.io/en/stable/webhooks.html#using-the-generic-api-integration
#
# we don't use the `ServiceHook` subsystem in this integration, because
# it is meant to pass the token in query params, and Read the Docs reference
# cited above instructs to send it as form data.
class ReadthedocsService < Service
  boolean_accessor :skip_tls_verification
  prop_accessor :project_url, :token

  validates :project_url, presence: true, public_url: true, if: :activated?
  validates :token, presence: true, if: :activated?

  def title
    'Read the Docs'
  end

  def description
    'Automate Read the Docs builds upon changes'
  end

  def self.to_param
    'readthedocs'
  end

  def self.supported_events
    %w(push)
  end

  def help
    'This service sends notifications about branch changes to Read the Docs
    to trigger new builds.

    To set up this service:

    - Refer to the [Read the Docs instructions](https://docs.readthedocs.io/en/stable/webhooks.html#using-the-generic-api-integration)
    - Paste the **Webhook URL** and the **token** into the fields below.
    '
  end

  def webhook_placeholder
    'https://readthedocs.org/api/v2/webhook/example-project/1/'
  end

  def execute(push_data)
    gitlab_branch = Gitlab::Git.branch_name(push_data[:ref])
    begin
      branch, topic = Gitlab::Mercurial.parse_gitlab_branch_for_hg(
        gitlab_branch)
    rescue StandardError
      # bookmark, TODO yes there could be a more semantic exception for that
      return
    end
    # We don't want to notify Read the Docs for not landed changesets
    # hence we filter out topics.

    # TODO we could also restrict to a given set of named branches. After
    # all only those that are configured on the Read the Docs side will
    # be useful. This would require a new field/property.
    return unless topic.nil?

    Gitlab::AppLogger.info(
      message: "readthedocs service/integration notifying",
      branches: branch,
      project_url: project_url
    )
    Gitlab::HTTP.post(project_url,
                      body: { branches: branch, token: token.strip },
                      verify: !skip_tls_verification?)
  end

  def fields
    [
      { type: 'text', name: 'project_url',
        title: 'Read the Docs project webhook URL',
        placeholder: webhook_placeholder,
        required: true },
      { type: 'password', name: 'token',
        title: 'The Read the Docs token',
        required: true },
      { type: 'checkbox', name: 'skip_tls_verification',
        title: 'Skip TLS/SSL verification' }
     ]
  end
end
