# frozen_string_literal: true

module Gitlab
  module Mercurial
    NULL_NODE = ('00' * 20).freeze

    SHORT_NODE_LAST_CHAR = 11 # [0..SHORT_NODE_LAST_CHAR] makes a length 12

    BookmarksUnsupportedError = Class.new(StandardError)

    class << self
      # Application-wide HGRC files, in a suitable form for the HGRCPATH
      # environment variable.
      def hgrc_path
        Gitlab.config.mercurial.hgrc.join(":")
      end

      def parse_gitlab_branch_for_hg(gitlab_branch)
        if gitlab_branch.start_with?('branch/')
          [gitlab_branch[7..], nil]
        elsif gitlab_branch.start_with?('topic/')
          branch, _slash, topic = gitlab_branch[6..].rpartition('/')
          [branch, topic]
        else
          raise BookmarksUnsupportedError, gitlab_branch
        end
      end

      # true if the given revision is a full hexadecimal changeset hash
      def full_hex_hash?(revision)
        revision =~ /\A\h{40}\z/
      end

      # Provide branchmap key for named branch and topic
      # Branchmap keys are displayed when using `hg heads`, and can
      # be used directly for some operations, notably to address remote
      # revisions.
      def branchmap_key(hg_branch, topic)
        if topic.nil?
          hg_branch
        else
          "#{hg_branch}:#{topic}"
        end
      end

      # Convert GitLab branch to corresponding branchmap key
      #
      # Branchmap keys are displayed when using `hg heads`, and can
      # be used directly for some operations, notably to address remote
      # revisions.
      def branchmap_key_for_gitlab_branch(gitlab_branch)
        self.branchmap_key(*self.parse_gitlab_branch_for_hg(gitlab_branch))
      end

      def parse_hg_archival(lines)
        arch = {}
        lines.each do |l|
          key, value = l.split(":", 2)
          arch[key.to_sym] = value.strip
        end
        arch
      end

      # dir_path: where `.hg_archival.txt` is expected to be
      #           the best is to provide a Pathname instance, but strings are
      #           accepted
      # returns nil if archival file does not exist, otherwise a Hash with
      #         symbol keys.
      def read_hg_archival(dir_path)
        path = Pathname(dir_path).join(".hg_archival.txt")
        return unless path.exist?

        parse_hg_archival(path.readlines)
      end
    end
  end
end
