# frozen_string_literal: true

require "spec_helper"

# rubocop:disable RSpec/FilePath
RSpec.describe Gitlab::Mercurial::HgitalyRepository, :seed_helper do
  include Gitlab::EncodingHelper
  include RepoHelpers
  include Gitlab::Popen

  before do
    stub_const('HGITALY_TEST_REPO_PATH', 'hgitaly-repo.git')
  end

  let(:repository) do
    repo = Gitlab::Mercurial::HgitalyRepository.new('default', HGITALY_TEST_REPO_PATH, '', 'group/project')
    repo.unit_tests_skip_hooks = true
    repo
  end

  # Same repository as :repository, seen as HgGitRepository, so that
  #
  #   1. we can use HgGitRepository as a reference for assertions on method
  #      results
  #   2. we have easy fixture preparation options
  let(:hg_git_repository) do
    repo = Gitlab::Mercurial::HgGitRepository.new('default', HGITALY_TEST_REPO_PATH, '', 'group/project')
    repo.unit_tests_skip_hooks = true
    repo
  end

  let(:repository_path) { File.join(TestEnv.repos_path, repository.relative_path).chomp('.git') + '.hg' }
  let(:user) { build(:user) }
  let(:non_empty_repository) do
    status = hg_git_repository.create_repository(namespace_fs_path: "group",
                                    in_subgroup: false)
    expect(status).to be(true)
    # early diagnostic for errors due to teardown problems:
    expect(hg_git_repository.empty?).to be(true)

    hg_git_repository.multi_action(user,
                      branch_name: 'branch/default',
                      message: "Commit 0",
                      author_email: "jdoe1@heptapod.test",
                      author_name: "John Doe1",
                      actions: [{ action: :create,
                                  file_path: 'foo',
                                  content: "foo0\n" }])
    hg_git_repository.expire_has_local_branches_cache
    expect(hg_git_repository.empty?).to be(false)

    repository # HgitalyRepository
  end

  let(:non_empty_repository_path) { repository_path }

  describe '#branch_names' do
    subject { non_empty_repository }

    after do
      subject.remove
    end

    it 'returns the correct list after repository seeding' do
      # long-term form of assertion
      expect(subject.branch_names).to eq(["branch/default"])
      # consistency with HgGitRepository
      expect(subject.branch_names).to eq(hg_git_repository.branch_names)
    end
  end

  describe '#branch_count' do
    # client: tests `count_branch_names`
    subject { non_empty_repository }

    after do
      subject.remove
    end

    it 'returns the correct count after repository seeding' do
      # long-term form of assertion
      expect(subject.branch_count).to eq(1)
      # consistency with HgGitRepository
      expect(subject.branch_count).to eq(hg_git_repository.branch_count)
    end
  end

  describe '#find_branch' do
    subject { non_empty_repository }

    after do
      subject.remove
    end

    it 'returns true for existing branch' do
      # long-term form of assertion
      expect(subject.find_branch("branch/default").name).to eq("branch/default")
      # consistency with HgGitRepository
      expect(hg_git_repository.find_branch("branch/default").name).to eq("branch/default")
    end

    it 'returns nil for non-existing branch' do
      # long-term form of assertion
      expect(subject.find_branch("branch_does_not_exists")).to eq(nil)
      # consistency with HgGitRepository
      expect(hg_git_repository.find_branch("branch_does_not_exists")).to eq(nil)
    end
  end

  describe '#tags' do
    subject { non_empty_repository }

    after do
      subject.remove
    end

    it 'returns the correct list after repository seeding' do
      subject.hg_tag(user, "v1.0", "default")
      # long-term form of assertion
      tags = subject.tags.map(&:name)
      expect(tags).to eq(["v1.0"])
      # consistency with HgGitRepository
      expect(tags).to eq(hg_git_repository.tags.map(&:name))
    end
  end

  describe '#tag_names' do
    subject { non_empty_repository }

    after do
      subject.remove
    end

    it 'returns the correct list after repository seeding' do
      subject.hg_tag(user, "v1.0", "default")
      # long-term form of assertion
      expect(subject.tag_names).to eq(["v1.0"])
      # consistency with HgGitRepository
      expect(subject.tag_names).to eq(hg_git_repository.tag_names)
    end
  end

  describe '#tag_count' do
    # client: tests `count_tag_names`
    subject { non_empty_repository }

    after do
      subject.remove
    end

    it 'returns the correct count after repository seeding' do
      subject.hg_tag(user, "v1.0", "default")
      # long-term form of assertion
      expect(subject.tag_count).to eq(1)
      # consistency with HgGitRepository
      expect(subject.tag_count).to eq(hg_git_repository.tag_count)
    end
  end

  describe '#local_branches' do
    subject! { non_empty_repository }

    let(:l_branches) { non_empty_repository.local_branches.map(&:name) }

    after do
      subject.remove
    end

    it 'returns the correct list after repository seeding' do
      # long-term form of assertion
      expect(l_branches).to eq(["branch/default"])
      # consistency with HgGitRepository
      expect(l_branches).to eq(hg_git_repository.local_branches.map(&:name))
    end
  end

  describe '#branches' do
    subject! { non_empty_repository }

    let(:branches) { non_empty_repository.branches.map(&:name) }

    after do
      subject.remove
    end

    it 'returns the correct list after repository seeding' do
      # long-term form of assertion
      expect(branches).to eq(["branch/default"])
      # consistency with HgGitRepository
      expect(branches).to eq(hg_git_repository.branches.map(&:name))
    end
  end

  describe '#root_ref' do
    # hgitaly: tests `default_branch_name`
    subject! { non_empty_repository }

    let(:d_branch) { non_empty_repository.root_ref }

    after do
      subject.remove
    end

    it 'returns the correct list after repository seeding' do
      # long-term form of assertion
      expect(d_branch).to eq("branch/default")
      # consistency with HgGitRepository
      expect(d_branch).to eq(hg_git_repository.root_ref)
    end
  end

  describe '#ref_exists?' do
    subject { non_empty_repository }

    after do
      subject.remove
    end

    it 'finds the default branch ref' do
      expect(subject.ref_exists?('refs/heads/branch/default')).to eq(true)
      # consistency with HgGitRepository
      expect(hg_git_repository.ref_exists?('refs/heads/branch/default')).to eq(true)
    end

    it 'returns false for an illegal tag name ref' do
      expect(subject.ref_exists?('refs/tags/.this-tag-name-is-illegal')).to eq(false)
      # consistency with HgGitRepository
      expect(hg_git_repository.ref_exists?('refs/tags/.this-tag-name-is-illegal')).to eq(false)
    end
  end

  describe "ref methods" do
    subject! { non_empty_repository }

    after do
      subject.remove
    end

    let(:hg_commit_sha) { subject.hgsha_from_rev('default') }
    let(:git_commit_sha) { hg_git_repository.commit.sha }
    let(:ref_path) { 'refs/pipelines/3476' }

    it "perform special refs operations in both repositories" do
      [hg_commit_sha, 'refs/heads/branch/default'].each do |target|
        subject.write_ref(ref_path, target)

        expect(subject.ref_exists?(ref_path)).to be(true)
        expect(subject.git_repository.ref_exists?(ref_path)).to be(true)

        expect(subject.git_repository.commit(ref_path).id)
          .to eq(git_commit_sha)
        expect(subject.commit(ref_path).id)
          .to eq(hg_commit_sha)

        expect { subject.delete_refs(ref_path) }.not_to raise_error
        expect(subject.ref_exists?(ref_path)).to be_falsey
        expect(subject.git_repository.ref_exists?(ref_path)).to be_falsey
      end
    end
  end

  describe('commit') do
    subject! { non_empty_repository }

    after do
      subject.remove
    end

    let(:hg_commit_sha) { subject.hgsha_from_rev('default') }
    let(:git_commit_sha) { hg_git_repository.commit.sha }

    it 'returns a correct Commit object' do
      commit = subject.commit(hg_commit_sha)
      expect(commit.id).to eq(hg_commit_sha)
      expect(commit.message).to eq('Commit 0')
      # making sure that the hack of switching repository on the fly
      # for VCS-qualified storage does not get propagated (would defeat its
      # very purpose
      expect(commit.instance_variable_get(:@repository)).to eq(subject)

      # fallback for leftover links to Git SHAs after migration from 'hg_git' to 'hg'
      # (see heptapod#421)
      fb_commit = subject.commit(git_commit_sha)
      expect(fb_commit.id).to eq(commit.id)
      expect(fb_commit.message).to eq(commit.message)

      # the fallback doesn't break semantics when revision is really unresolvable
      expect(subject.commit("unresolvable")).to be_nil
    end
  end

  describe "unknown SHAs'" do
    subject! { non_empty_repository }

    after do
      subject.remove
    end

    let(:hg_commit_sha) { subject.hgsha_from_rev('default') }
    let(:git_commit_sha) { hg_git_repository.commit.sha }
    let(:unknown_sha) { '12cafe34' * 5 }

    it 'method ancestor? returns false for unknown hashes' do
      unknown_sha = '12cafe34' * 5
      # for reference
      expect(
        hg_git_repository.ancestor?(unknown_sha, git_commit_sha)).to be_falsey

      # actual test
      expect(subject.ancestor?(unknown_sha, hg_commit_sha)).to be_falsey
    end

    it 'method count_commits_between returns 0 for unknown hashes' do
      # for reference
      expect(hg_git_repository.count_commits(ref: unknown_sha)).to eq(0)
      expect(
        hg_git_repository.count_commits(
          from: unknown_sha, to: git_commit_sha)).to eq(0)

      # actual test
      expect(subject.count_commits(ref: unknown_sha)).to eq(0)
      expect(subject.count_commits(
               from: unknown_sha, to: hg_commit_sha)).to eq(0)
    end

    it 'gives empty arrays in the associated Commit.between' do
      # for reference
      [
        [unknown_sha, git_commit_sha],
        [git_commit_sha, unknown_sha],
        [unknown_sha, unknown_sha]
      ].each do |pair|
        expect(Gitlab::Git::Commit.between(hg_git_repository, *pair)).to eq([])
      end

      # actual test
      [
        [unknown_sha, hg_commit_sha],
        [hg_commit_sha, unknown_sha],
        [unknown_sha, unknown_sha]
      ].each do |pair|
        expect(Gitlab::Git::Commit.between(subject, *pair)).to eq([])
      end
    end

    it 'gives empty lists in the associated Commit.batch_by_oids' do
      # for reference
      expect(
        Gitlab::Git::Commit.batch_by_oid(hg_git_repository, [unknown_sha])
      ).to eq([])

      # actual test
      expect(
        Gitlab::Git::Commit.batch_by_oid(subject, [unknown_sha])
      ).to eq([])
    end
  end

  describe "archive_sha" do
    subject { non_empty_repository }

    after do
      subject.remove
    end

    it "is the Mercurial SHA" do
      hg_sha = subject.hgsha_from_rev('default')

      expect(subject.archive_sha(hg_sha)).to eq(hg_sha)
    end
  end

  describe "commit display id" do
    subject { non_empty_repository }

    after do
      subject.remove
    end

    it "is the Mercurial commit ID" do
      hg_sha = subject.hgsha_from_rev('default')

      expect(subject.commit_id_to_display_id(hg_sha)).to eq(hg_sha)
      expect(subject.commit_id_from_display_id(hg_sha)).to eq(hg_sha)

      commit = subject.commit(hg_sha)
      expect(commit.hpd_display_id).to eq(hg_sha)
    end
  end

  describe '#branch_names_contains_sha' do
    subject! { non_empty_repository }

    let(:git_commit_sha) { hg_git_repository.commit.sha }
    let(:hg_commit_sha) { subject.hgsha_from_rev('default') }

    after do
      subject.remove
    end

    it 'return correct branch names' do
      expect(subject.branch_names_contains_sha(hg_commit_sha)).to eq(["branch/default"])
      # consistency with HgGitRepository
      expect(hg_git_repository.branch_names_contains_sha(git_commit_sha)).to eq(["branch/default"])
    end
  end

  describe '#tag_names_contains_sha' do
    subject! { non_empty_repository }

    after do
      subject.remove
    end

    it 'return correct tag names' do
      hg_commit_sha = subject.hgsha_from_rev('default')
      git_commit_sha = hg_git_repository.commit.sha
      subject.hg_tag(user, "v1.0", "default")
      expect(subject.tag_names_contains_sha(hg_commit_sha)).to eq(["v1.0"])
      # consistency with HgGitRepository
      expect(hg_git_repository.tag_names_contains_sha(git_commit_sha)).to eq(["v1.0"])
    end
  end

  describe '#count_commits' do
    subject! { non_empty_repository }

    let(:prepare_repo) do
      # repo structure:
      #
      #         2 (topic/default/feature)
      #         |
      #         1 (branch/default)
      #         |
      #         0
      #
      subject.multi_action(user,
              branch_name: 'branch/default',
              message: "Commit 1",
              author_email: "jdoe1@heptapod.test",
              author_name: "John Doe1",
              actions: [{ action: :create,
                          file_path: "foo1",
                          content: "foo1\n" }])
      subject.multi_action(user,
              branch_name: 'topic/default/feature',
              start_branch_name: 'branch/default',
              message: "Commit 2",
              author_email: "jdoe1@heptapod.test",
              author_name: "John Doe1",
              actions: [{ action: :create,
                          file_path: "foo2",
                          content: "foo2\n" }])
    end

    before do
      prepare_repo
    end

    after do
      subject.remove
    end

    context "with revision" do
      it "returns the number of commits with target branch/default" do
        options = { ref: "branch/default" }

        expect(subject.count_commits(options)).to eq(2)
        # consistency with HgGitRepository
        expect(hg_git_repository.count_commits(options)).to eq(2)
      end

      it "returns the number of commits with target topic/default/feature" do
        options = { ref: "topic/default/feature" }

        expect(subject.count_commits(options)).to eq(3)
        # consistency with HgGitRepository
        expect(hg_git_repository.count_commits(options)).to eq(3)
      end
    end

    context "with revision range" do
      it "returns the same thing as the `only` revset in Mercurial" do
        options = { from: "branch/default",
                    to: "topic/default/feature",
                    left_right: true }
        # left_right returns [counts of to..from, counts of from..to]
        expect(hg_git_repository.count_commits(options)).to eq([0, 1])
        expect(subject.count_commits(options)).to eq([0, 1])

        # additional commit on branch/default
        # that makes an ancestor of branch/default that's not ancestor
        # of topic/default/feature
        subject.multi_action(user,
              branch_name: 'branch/default',
              start_branch_name: 'branch/default',
              message: "Commit 3",
              author_email: "jdoe1@heptapod.test",
              author_name: "John Doe1",
              actions: [{ action: :create,
                          file_path: "foo2",
                          content: "foo2\n" }])

        expect(hg_git_repository.count_commits(options)).to eq([1, 1])
        expect(subject.count_commits(options)).to eq([1, 1])
      end
    end

    context "with all" do
      it "returns the number of commits in the whole repository" do
        options = { all: true }

        expect(subject.count_commits(options)).to eq(3)
        # consistency with HgGitRepository
        expect(hg_git_repository.count_commits(options)).to eq(3)
      end
    end

    context 'with max_count' do
      it 'returns the number of commits up to the passed limit' do
        options = { max_count: 1, ref: "branch/default" }

        expect(repository.count_commits(options)).to eq(1)
        # consistency with HgGitRepository
        expect(hg_git_repository.count_commits(options)).to eq(1)
      end
    end
  end

  describe '#last_commit_for_path' do
    subject! { non_empty_repository }

    after do
      subject.remove
    end

    let(:git_commit_sha) { hg_git_repository.commit.sha }
    let(:hg_commit_sha) { subject.hgsha_from_rev('default') }

    it 'returns the expected commit' do
      expect(subject.last_commit_for_path('branch/default', 'foo').id
            ).to eq(hg_commit_sha)
      expect(hg_git_repository.last_commit_for_path('branch/default', 'foo').id
            ).to eq(git_commit_sha)

      # file that does not exist
      expect(subject.last_commit_for_path('branch/default', 'nope')).to be_nil
      expect(hg_git_repository.last_commit_for_path(
               'branch/default', 'nope')).to be_nil

      # subdirectory matching
      subject.multi_action(
        user,
        branch_name: 'branch/default',
        message: "Commit 1",
        author_email: "jdoe1@heptapod.test",
        author_name: "John Doe1",
        actions: [{ action: :create,
                    file_path: "sub/foo1",
                    content: "foo1\n" }]
      )
      new_git_sha = hg_git_repository.commit.sha
      expect(hg_git_repository.last_commit_for_path('branch/default', 'sub').id
            ).to eq(new_git_sha)
      expect(subject.last_commit_for_path('branch/default', 'sub').id
            ).to eq(subject.sha_from_gitsha(new_git_sha))
    end
  end

  # Tell if two blobs have identical instance variables
  #
  # Blobs don't implement equality, and it could possibly be a problem
  # with partially loaded data if they did.
  # Hence `expect(blob1).to eq(blob2)` fails in any case unless they
  # are the very same Ruby objects.
  def blobs_equal?(blob1, blob2)
    %i[id mode name path size commit_id data binary].all? do |attr|
      blob1.send(attr) == blob2.send(attr)
    end
  end

  describe '#diverging_commit_count' do
    subject! { non_empty_repository }

    def hg_commit(cid, topic: nil)
      if topic
        subject.hg_call(['topic', topic])
      end

      subject.write_file(subject.hg_full_path + "/file#{cid}", "line\n")
      subject.hg_call(["add", "file#{cid}"])
      subject.hg_call(['ci', '-m', "Commit #{cid}", '--config',
                       'experimental.single-head-per-branch=False'],
                      user: user, for_write: true)
      subject.hg_git_invalidate_maps!
    end

    let(:prepare_repo) do
      subject.hg_call(%w[up 0])
      hg_commit(1, topic: 'feature')
      hg_commit(2)
      hg_commit(3)
      subject.hg_call(%w[up 0])
      hg_commit(4)
      hg_commit(5)
    end

    before do
      prepare_repo
    end

    after do
      subject.remove
    end

    it 'counts 0 for the same branch' do
      expect(subject.diverging_commit_count('branch/default', 'branch/default', max_count: 1000)).to eq([0, 0])
      # consistency with HgGitRepository
      expect(hg_git_repository.diverging_commit_count('branch/default', 'branch/default', max_count: 1000)).to eq([0, 0])
    end

    context 'max count does not truncate results' do
      it 'returns the correct count bounding at max_count' do
        expect(subject.diverging_commit_count(
                 'branch/default', 'topic/default/feature', max_count: 1000
               )).to eq([2, 3])
        # consistency with HgGitRepository
        expect(hg_git_repository.diverging_commit_count(
                 'branch/default', 'topic/default/feature', max_count: 1000
               )).to eq([2, 3])
      end
    end

    context 'max count truncates results' do
      it 'returns the correct count bounding at max_count' do
        max_count = 4
        resp = subject.diverging_commit_count('branch/default', 'topic/default/feature', max_count: max_count)
        expect(resp[0] + resp[1]).to eq(max_count)
        # consistency with HgGitRepository
        resp = hg_git_repository.diverging_commit_count('branch/default', 'topic/default/feature', max_count: max_count)
        expect(resp[0] + resp[1]).to eq(max_count)
      end
    end
  end

  describe '#merge_base' do
    subject! { non_empty_repository }

    def hg_commit(cid, topic: nil)
      if topic
        subject.hg_call(['topic', topic])
      end

      subject.write_file(subject.hg_full_path + "/file#{cid}", "line\n")
      subject.hg_call(["add", "file#{cid}"])
      subject.hg_call(['ci', '-m', "Commit #{cid}", '--config',
                       'experimental.single-head-per-branch=False'],
                      user: user, for_write: true)
      subject.hg_git_invalidate_maps!
    end

    let(:prepare_repo) do
      subject.hg_call(%w[up 0])
      hg_commit(1, topic: 'feature')
      subject.hg_call(%w[up 0])
      hg_commit(2)
    end

    before do
      prepare_repo
    end

    after do
      subject.remove
    end

    it 'returns the merge base for two commits' do
      hg_sha0 = subject.hgsha_from_rev('0')
      expect(
        subject.merge_base('topic/default/feature', 'branch/default')
      ).to eq hg_sha0
    end
  end

  describe '#list_commits_by_ref_name' do
    subject! { non_empty_repository }

    def hg_commit(cid, topic: nil)
      if topic
        subject.hg_call(['topic', topic])
      end

      subject.write_file(subject.hg_full_path + "/file#{cid}", "line\n")
      subject.hg_call(["add", "file#{cid}"])
      subject.hg_call(['ci', '-m', "Commit #{cid}", '--config',
                       'experimental.single-head-per-branch=False'],
                      user: user, for_write: true)
      subject.hg_git_invalidate_maps!
    end

    # repo structure:
    #
    #    2 (topic/default/feature)
    #    |
    #    | 1 (branch/default)
    #    |/
    #    0
    #
    let(:prepare_repo) do
      subject.hg_call(%w[up 0])
      hg_commit(1)
      subject.hg_call(%w[up 0])
      hg_commit(2, topic: 'feature')
    end

    before do
      prepare_repo
    end

    after do
      subject.remove
    end

    it 'return commits corresponding the given ref names' do
      hg_sha1 = subject.hgsha_from_rev('1')
      hg_sha2 = subject.hgsha_from_rev('2')
      hg_git_repository.hg_git_invalidate_maps!
      git_sha1 = hg_git_repository.sha_from_hgsha(hg_sha1)
      git_sha2 = hg_git_repository.sha_from_hgsha(hg_sha2)

      response = subject.list_commits_by_ref_name(['branch/default', 'topic/default/feature', 'does_not_exists'])
      expect(response.keys.count).to eq 2
      expect(response['topic/default/feature'].id).to eq hg_sha2
      expect(response['branch/default'].id).to eq hg_sha1
      expect(response).not_to have_key 'nonexistent'
      # consistency with HgGitRepository
      response = hg_git_repository.list_commits_by_ref_name(['branch/default', 'topic/default/feature', 'does_not_exists'])
      expect(response.keys.count).to eq 2
      expect(response['topic/default/feature'].id).to eq git_sha2
      expect(response['branch/default'].id).to eq git_sha1
      expect(response).not_to have_key 'nonexistent'
    end
  end

  describe '#log' do
    subject! { non_empty_repository }

    def hg_commit(cid)
      subject.write_file(subject.hg_full_path + "/file#{cid}", "line\n")
      subject.hg_call(["add", "file#{cid}"])
      subject.hg_call(['ci', '-m', "Commit #{cid}", '--config',
                       'experimental.single-head-per-branch=False'],
                      user: user, for_write: true)
      subject.hg_git_invalidate_maps!
    end

    # repo structure:
    #
    #    o 1 (branch/default)
    #    |
    #    0
    #
    let(:prepare_repo) do
      subject.hg_call(%w[up 0])
      hg_commit(1)
    end

    before do
      prepare_repo
    end

    after do
      subject.remove
    end

    it 'return commits for the given ref name' do
      hg_sha0 = subject.hgsha_from_rev('0')
      hg_sha1 = subject.hgsha_from_rev('1')
      hg_git_repository.hg_git_invalidate_maps!
      git_sha0 = hg_git_repository.sha_from_hgsha(hg_sha0)
      git_sha1 = hg_git_repository.sha_from_hgsha(hg_sha1)

      resp = subject.log({ ref: 'branch/default' }).map(&:id)
      expect(resp).to eq([hg_sha1, hg_sha0])
      # consistency with HgGitRepository
      resp = hg_git_repository.log({ ref: 'branch/default' }).map(&:id)
      expect(resp).to eq([git_sha1, git_sha0])
    end
  end
end
