# frozen_string_literal: true

require 'spec_helper'

RSpec.describe Gitlab::Mercurial do
  before do
    cls = Class.new(Object)
    cls.send(:include, Gitlab::Popen)
    @popen = cls.new
  end

  describe 'config.mercurial.hgrc' do
    it 'points to existing files' do
      Gitlab.config.mercurial.hgrc.each do |path|
        expect(File.exist?(path)).to be(true)
      end
    end
  end

  describe '.hgrc_path' do
    let(:hg_env) { { 'HGRCPATH' => described_class.hgrc_path } }
    let(:hg_exe) { Gitlab.config.mercurial.bin_path }

    it 'activates the wished extensions' do
      cmd = [hg_exe, 'debugextensions']
      out, status = @popen.popen(cmd, nil, hg_env)
      expect(status).to eq(0)
      expect(out).to include("heptapod (untested!)")
    end

    it 'sets the required local settings' do
      cmd = [Gitlab.config.mercurial.bin_path, 'config', 'heptapod']
      out, status = @popen.popen(cmd, '/', hg_env)
      expect(status).to eq(0)
      expect(out).to include("gitlab-internal-api-secret-file")
    end
  end

  describe '.parse_hg_archival' do
    subject { described_class.parse_hg_archival }

    it 'is correct on a real-life input' do
      expect(described_class.parse_hg_archival(
               ["repo: 67091225e50d1e8d4ddd1d090f3255065c5ac5ee\n",
                "node: 11c5a0a148c91cf972e19eff87873671d4b79e07\n",
                "branch: heptapod-0-16\n",
                "tag: heptapod-0.16.0\n"]
             )).to eq({ repo: "67091225e50d1e8d4ddd1d090f3255065c5ac5ee",
                        node: "11c5a0a148c91cf972e19eff87873671d4b79e07",
                        branch: "heptapod-0-16",
                        tag: "heptapod-0.16.0" })
    end

    it 'handles the colons in values' do
      expect(described_class.parse_hg_archival(["key:value:has:colons"])
            ).to eq({ key: "value:has:colons" })
    end
  end

  describe '.read_hg_archival' do
    let(:dirpath) do
      # all subdirs of tmp/tests get automatically removed
      common = Pathname(TestEnv::TMP_TEST_PATH).join("spec_hg_archival")
      common.mkdir unless common.exist?
      dirpath = common.join(Random.rand(1 << 32).to_s)
      dirpath.mkdir
      dirpath
    end

    it 'works with a Pathname instance' do
      dirpath.join('.hg_archival.txt').write("node: 12cafe34")
      expect(described_class.read_hg_archival(dirpath)
            ).to eq({ node: "12cafe34" })
    end

    it 'works with path as a string' do
      dirpath.join('.hg_archival.txt').write("tag: heptapod-0.1.0.pre")
      expect(described_class.read_hg_archival(dirpath.to_s)
            ).to eq({ tag: "heptapod-0.1.0.pre" })
    end

    it 'returns nil if metadata file is missing' do
      expect(described_class.read_hg_archival(dirpath)).to be(nil)
    end
  end

  describe 'branchmap_key_for_gitlab_branch' do
    it 'is correct for named branches' do
      expect(described_class.branchmap_key_for_gitlab_branch(
               'branch/heptapod')).to eq('heptapod')
    end
    it 'is correct for named branches' do
      expect(described_class.branchmap_key_for_gitlab_branch(
               'topic/stable/urgent-bugfix')).to eq('stable:urgent-bugfix')
    end
  end
end
